import React from 'react';
import styled from 'styled-components';
import PropTypes from 'prop-types';
import { Transition } from 'react-transition-group';
import { color, width, height, compose } from 'styled-system';
import { themeGet } from '@styled-system/theme-get';
import { DialogOverlay, DialogContent } from '@reach/dialog';
import { Box, CloseButton, deprecatedColorValue, Flex } from 'pcln-design-system';
const OVERLAY_ANIMATION = transitionState => `
  opacity: 0;
  transition: opacity .5s cubic-bezier(0.50, 0.00, 0.25, 1.00);
  ${transitionState === 'entering' ? `opacity: 0;` : ''}
  ${transitionState === 'entered' ? `opacity: 1;` : ''}
  ${transitionState === 'exiting' ? `opacity: 0;` : ''}
  ${transitionState === 'exited' ? `opacity: 0;` : ''}
`;
const DIALOG_ANIMATION = transitionState => `
  transform: scale(0.5);
  transition: transform .5s cubic-bezier(0.50, 0.00, 0.25, 1.00);
  ${transitionState === 'entering' ? `transform: scale(0.5);` : ''}
  ${transitionState === 'entered' ? `transform: scale(1);` : ''}
  ${transitionState === 'exiting' ? `transform: scale(0.5);` : ''}
  ${transitionState === 'exited' ? `transform: scale(0.5);` : ''}
`;
const getAnimation = _ref => {
  let {
    transitionstate,
    defaultAnimation,
    customAnimation = null
  } = _ref;
  return typeof customAnimation === 'function' ? customAnimation(transitionstate) : defaultAnimation(transitionstate);
};
const Overlay = styled(DialogOverlay).withConfig({
  displayName: "Modal__Overlay",
  componentId: "sc-rzof3m-0"
})(["background-color:rgba(0,0,0,0.7);bottom:0;left:0;overflow-y:auto;position:fixed;right:0;top:0;", " *{box-sizing:border-box;}", ";"], props => `
    z-index: ${props.zindex || 100};
    font-family: ${props.theme.font};
    line-height: ${props.theme.lineHeights?.standard};
    font-weight: ${props.theme.fontWeights?.medium};
  `, getAnimation);
const Dialog = styled(DialogContent).withConfig({
  displayName: "Modal__Dialog",
  componentId: "sc-rzof3m-1"
})(["position:relative;margin-left:auto;margin-right:auto;box-shadow:", ";border-radius:", ";&:focus{outline:none;}", " ", " ", " ", ""], props => props.theme.shadows['overlay-xl'], _ref2 => {
  let {
    borderRadius
  } = _ref2;
  return themeGet(`borderRadii.${borderRadius}`);
}, props => !props.fullScreen ? `
    max-height: 100vh;
    max-width: calc(100vw - 32px);
  ` : `
    width: 100vw;
    height: 100vh;
  `, props => props.enableoverflow && `
    max-height: none;
  `, getAnimation, props => compose(color, width, height)(props));
const HeaderWrapper = styled.div.withConfig({
  displayName: "Modal__HeaderWrapper",
  componentId: "sc-rzof3m-2"
})(["flex-shrink:0;"]);
const FloatCloseButton = styled(CloseButton).withConfig({
  displayName: "Modal__FloatCloseButton",
  componentId: "sc-rzof3m-3"
})(["position:absolute;top:16px;right:16px;z-index:2;svg{vertical-align:top;}&:focus{background-color:", ";outline:none;}"], props => props.theme.colors.borderGray);
const ContentWrapper = styled(Box).withConfig({
  displayName: "Modal__ContentWrapper",
  componentId: "sc-rzof3m-4"
})(["position:relative;", ""], props => {
  if (!props.enableoverflow) {
    return `
        overflow-y: auto;
        overflow-x: hidden;
        height: 100%;
        -webkit-overflow-scrolling: touch;
      `;
  }
});
const OverlayWrapper = styled.div.withConfig({
  displayName: "Modal__OverlayWrapper",
  componentId: "sc-rzof3m-5"
})(["display:table;height:100%;width:100%;"]);
const DialogWrapper = styled(Box).withConfig({
  displayName: "Modal__DialogWrapper",
  componentId: "sc-rzof3m-6"
})(["display:table-cell;vertical-align:", ";position:relative;height:100%;", ";"], _ref3 => {
  let {
    verticalAlignment
  } = _ref3;
  return verticalAlignment;
}, props => props.enableoverflow && `
    padding-top: 24px;
    padding-bottom: 24px;
  `);
const DialogInnerWrapper = styled(Flex).withConfig({
  displayName: "Modal__DialogInnerWrapper",
  componentId: "sc-rzof3m-7"
})(["position:relative;height:100%;"]);
const Modal = _ref4 => {
  let {
    ariaLabel,
    ariaLabelledBy,
    bg,
    borderRadius,
    children,
    className,
    dialogAnimation,
    disableCloseButton,
    enableOverflow,
    fullScreen,
    header,
    height,
    imgMode,
    isOpen,
    onClose,
    overlayAnimation,
    timeout,
    verticalAlignment,
    width,
    zIndex
  } = _ref4;
  let dialogHeight = height;
  if (enableOverflow) {
    dialogHeight = null;
  } else if (fullScreen) {
    dialogHeight = '100vh';
  }
  return /*#__PURE__*/React.createElement(Transition, {
    in: isOpen,
    unmountOnExit: true,
    timeout: timeout
  }, state => /*#__PURE__*/React.createElement(Overlay, {
    onDismiss: onClose,
    zindex: zIndex,
    transitionstate: state,
    initialFocusRef: null,
    defaultAnimation: OVERLAY_ANIMATION,
    customAnimation: overlayAnimation
  }, /*#__PURE__*/React.createElement(OverlayWrapper, null, /*#__PURE__*/React.createElement(DialogWrapper, {
    enableoverflow: enableOverflow,
    verticalAlignment: verticalAlignment
  }, /*#__PURE__*/React.createElement(Dialog, {
    width: fullScreen ? '100vw' : width,
    bg: bg,
    borderRadius: borderRadius,
    height: dialogHeight,
    transitionstate: state,
    className: className,
    enableoverflow: enableOverflow ? 'true' : null,
    defaultAnimation: DIALOG_ANIMATION,
    customAnimation: dialogAnimation,
    fullScreen: fullScreen,
    "data-testid": "dialog-content",
    "aria-label": ariaLabel,
    "aria-labelledby": ariaLabelledBy
  }, /*#__PURE__*/React.createElement(DialogInnerWrapper, {
    flexDirection: "column"
  }, header && /*#__PURE__*/React.createElement(HeaderWrapper, null, header), /*#__PURE__*/React.createElement(ContentWrapper, {
    borderRadius: borderRadius,
    p: imgMode ? 0 : 16,
    header: header,
    enableoverflow: enableOverflow
  }, enableOverflow && !disableCloseButton && /*#__PURE__*/React.createElement(FloatCloseButton, {
    "data-testid": "pcln-modal-close",
    header: header,
    onClick: onClose
  }), children)))))));
};
Modal.defaultProps = {
  bg: 'white',
  borderRadius: 'xl',
  dialogAnimation: null,
  disableCloseButton: false,
  enableOverflow: false,
  header: null,
  height: 420,
  isOpen: false,
  overlayAnimation: null,
  timeout: 500,
  verticalAlignment: 'middle'
};
const validateAriaProps = (props, propName, componentName) => {
  if (!props.ariaLabel && !props.ariaLabelledBy) {
    return new Error(`A <${componentName}> must have either an 'ariaLabel' or 'ariaLabelledBy' prop.`);
  }
};
Modal.propTypes = {
  ...width.propTypes,
  ...height.propTypes,
  ariaLabel: validateAriaProps,
  ariaLabelledBy: validateAriaProps,
  bg: deprecatedColorValue(),
  className: PropTypes.string,
  dialogAnimation: PropTypes.func,
  disableCloseButton: PropTypes.bool,
  enableOverflow: PropTypes.bool,
  fullScreen: PropTypes.bool,
  header: PropTypes.any,
  headerBg: PropTypes.string,
  imgMode: PropTypes.bool,
  isOpen: PropTypes.bool,
  onClose: PropTypes.func,
  overlayAnimation: PropTypes.func,
  timeout: PropTypes.number,
  title: PropTypes.string,
  verticalAlignment: PropTypes.string,
  zIndex: PropTypes.number
};
export default Modal;